package com.yihu.wlyy.web.doctor.patient;

import com.yihu.wlyy.entity.doctor.team.sign.SignPatientLabelInfo;
import com.yihu.wlyy.service.app.label.SignPatientLabelInfoService;
import com.yihu.wlyy.web.BaseController;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 患者所属标签控制类
 * <p>
 * Created by lyr on 2016/10/9.
 */
@RestController
@RequestMapping(value = "/doctor/patient_label_info")
public class SignPatientLabelInfoController extends BaseController {

    @Autowired
    SignPatientLabelInfoService labelInfoService;

    /**
     * 查询居民信息
     *
     * @param patient 患者
     * @return
     */
    @RequestMapping(value = "/patient")
    @ResponseBody
    public String getPatient(String patient){
        try{
            if(StringUtils.isEmpty(patient)){
                return error(-1,"居民不能为空");
            }
            JSONObject p = labelInfoService.getPatient(patient);
            return write(200,"查询成功","data",p);
        }catch (Exception e){
            e.printStackTrace();
            return error(-1,"查询失败");
        }
    }

    /**
     * 查询某个团队的某个标签下的患者信息
     *
     * @param labelCode 标签code
     * @param labelType 标签类型
     * @param teamCode  团队code
     * @param page      第几页
     * @param pagesize  页大小
     * @return
     */
    @RequestMapping(value = "/patients_by_label")
    public String getPatientInfoByLabel(String labelCode, String labelType, Long teamCode, int page, int pagesize) {
        try {
            if (StringUtils.isEmpty(labelCode)) {
                return error(-1, "标签cdoe不能为空");
            }
            if (StringUtils.isEmpty(labelType)) {
                return error(-1, "标签类型不能为空");
            }
            if (StringUtils.isEmpty(String.valueOf(teamCode)) || teamCode < 1) {
                return error(-1, "团队cdoe不能为空");
            }

            page = page - 1;
            JSONArray result = labelInfoService.getPatientByLabel(getUID(), labelCode, labelType, teamCode, page, pagesize);

            return write(200, "查询成功", "data", result);
        } catch (Exception e) {
            return error(-1, "查询失败");
        }
    }

    /**
     * 获取团队下的居民
     *
     * @param teamCode
     * @param page
     * @param pagesize
     * @return
     */
    @RequestMapping(value = "/patients_by_team")
    public String getPatientInfoByTeam(Long teamCode,
                                       @RequestParam(required = false) String exLabelCode,
                                       @RequestParam(required = false) String exLabelType,
                                       int page, int pagesize) {
        try {
            if (teamCode == null || teamCode < 1) {
                return error(-1, "团队cdoe不能为空");
            }

            if (!StringUtils.isEmpty(exLabelCode) && StringUtils.isEmpty(exLabelType)) {
                return error(-1, "过滤标签参数不为空时过滤标签类型不能为空");
            }

            page = page - 1;
            JSONArray result = labelInfoService.getPatientByTeamCode(getUID(), teamCode, exLabelCode,exLabelType,page, pagesize);

            return write(200, "查询成功", "data", result);
        } catch (Exception e) {
            return error(-1, "查询失败");
        }
    }

    /**
     * 查询某个团队的某个标签类型下各标签患者数
     *
     * @param labelType 标签类型
     * @param teamCode  团队code
     * @return
     */
    @RequestMapping(value = "/label_patient_amount")
    public String getPatientAmountByLabelType(String labelType, @RequestParam(required = false) Long teamCode) {
        try {
            if (StringUtils.isEmpty(labelType)) {
                return error(-1, "标签类型不能为空");
            }

            if (teamCode == null) {
                teamCode = 0L;
            }

            JSONArray result = labelInfoService.getPatientAmountByLabelType(getUID(), labelType, teamCode);

            return write(200, "查询成功", "data", result);
        } catch (Exception e) {
            return error(-1, "查询失败");
        }
    }

    /**
     * 患者标签更新
     *
     * @param patient 患者
     * @param health  健康情况
     * @param disease 疾病情况
     * @param custom  自定义情况
     * @return
     */
    @RequestMapping(value = "/patient_label_update")
    public String setPatientLabels(String patient, String health, String disease, String custom) {
        try {
            if (StringUtils.isEmpty(patient)) {
                return error(-1, "患者不能为空");
            }
            if (StringUtils.isEmpty(health)) {
                return error(-1, "健康情况标签不能为空");
            }

            int result = labelInfoService.setPatientLabels(patient, "", "", health, disease, custom);

            if (result == 0) {
                return error(-1, "查不到患者信息");
            }
            if (result == -1) {
                return error(-1, "标签不存在");
            }
            if (result == -2) {
                return error(-1, "健康情况标签不能为空");
            }
            if (result == 1) {
                return write(200, "更新成功");
            } else {
                return error(-1, "更新失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "更新失败");
        }
    }

    /**
     * 查询某个患者的某类标签
     *
     * @param patient   患者
     * @param labelType 标签类型
     * @return
     */
    @RequestMapping(value = "/patient_label")
    public String getPatientLabelByLabelType(String patient, @RequestParam(required = false) String labelType) {
        try {
            if (StringUtils.isEmpty(patient)) {
                return error(-1, "患者不能为空");
            }

            List<SignPatientLabelInfo> labels = labelInfoService.getPatientLabelByLabelType(patient, labelType);

            return write(200, "查询成功", "data", new JSONArray(labels));
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "查询失败");
        }
    }

    /**
     * 查询某个患者的某类标签
     *
     * @param filter 过滤字段
     * @return
     */
    @RequestMapping(value = "/patient_search")
    public String searchPatientByNameOrLabel(String filter,
                                             @RequestParam(required = false) String labelCode,
                                             @RequestParam(required = false) String labelType,
                                             @RequestParam(required = false) Long teamCode,
                                             @RequestParam(required = false) String exLabelCode,
                                             @RequestParam(required = false) String exLabelType,
                                             int page, int pagesize) {
        try {
            if (StringUtils.isEmpty(filter)) {
                return error(-1, "搜索字段不能为空");
            }

            if (!StringUtils.isEmpty(labelCode) && StringUtils.isEmpty(labelType)) {
                return error(-1, "标签参数不为空时标签类型不能为空");
            }

            if (!StringUtils.isEmpty(exLabelCode) && StringUtils.isEmpty(exLabelType)) {
                return error(-1, "过滤标签参数不为空时过滤标签类型不能为空");
            }

            if (teamCode == null) {
                teamCode = 0L;
            }

            page = page - 1;

            JSONArray result = labelInfoService.searchPatientByNameOrLabel(getUID(), filter, labelCode, labelType, teamCode, exLabelCode, exLabelType, page, pagesize);

            return write(200, "查询成功", "data", result);
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "查询失败");
        }
    }

    /**
     * 添加居民到某个标签
     *
     * @param patient   患者
     * @param labelCode 标签code
     * @param labelType 标签类型
     * @return
     */
    @RequestMapping(value = "/patient_label_add")
    public String addPatientLabel(String patient, String labelCode, String labelType) {
        try {
            if (StringUtils.isEmpty(String.valueOf(patient))) {
                return error(-1, "患者不能为空");
            }
            if (StringUtils.isEmpty(labelCode)) {
                return error(-1, "标签cdoe不能为空");
            }
            if (StringUtils.isEmpty(labelType)) {
                return error(-1, "标签类型不能为空");
            }

            int result = labelInfoService.addPatientsLabel(patient, labelCode, labelType);

            if (result == -1) {
                return error(-1, "患者不存在");
            } else if (result == -2) {
                return error(-1, "患者已在此标签");
            } else if (result == -3) {
                return error(-1, "标签不存在");
            } else if (result == 1) {
                return error(200, "添加成功");
            } else {
                return error(-1, "添加失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "添加失败");
        }
    }

    /**
     * 新增标签并分配患者
     *
     * @param labelName 标签名称
     * @param teamCode  团队Code
     * @param patients  患者
     * @return
     */
    @RequestMapping(value = "/label_and_patients_add")
    public String addLabelAndPatient(String labelName, Long teamCode, @RequestParam(required = false) String patients) {
        try {
            if (StringUtils.isEmpty(labelName)) {
                return error(-1, "标签名称不能为空");
            }
            if (teamCode == null || teamCode < 1) {
                return error(-1, "团队code不能为空");
            }

            if (labelName.equals("孕产妇") || labelName.equals("儿童")) {
                return error(-1, "该标签名称已存在");
            }

            int result = labelInfoService.addLabelAndPatient(labelName, getUID(), teamCode, patients);

            if (result != 1) {
                return error(-1, "添加失败");
            }

            return write(200, "添加成功");
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "添加失败");
        }
    }

    /**
     * 删除居民某个标签
     *
     * @param patient   患者
     * @param labelCode 标签code
     * @param labelType 标签类型
     * @return
     */
    @RequestMapping(value = "/patient_label_delete")
    public String deletePatientLabel(String patient, String labelCode, String labelType) {
        try {
            if (StringUtils.isEmpty(String.valueOf(patient))) {
                return error(-1, "患者不能为空");
            }
            if (StringUtils.isEmpty(labelCode)) {
                return error(-1, "标签cdoe不能为空");
            }
            if (StringUtils.isEmpty(labelType)) {
                return error(-1, "标签类型不能为空");
            }

            int result = labelInfoService.deltePatientsLabel(patient, labelCode, labelType);

            if (result == -1) {
                return error(-1, "患者没有此标签");
            } else if (result == 1) {
                return error(200, "添加成功");
            } else {
                return error(-1, "删除失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "删除失败");
        }
    }

    @RequestMapping(value = "/disease_to_label")
    public String diseaseToLabel() {
        try {
            if (labelInfoService.diseaseToLabel()) {
                return write(200, "转换成功");
            } else {
                return error(-1, "转换失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "转换失败");
        }
    }

    @RequestMapping(value = "/group_to_label")
    public String groupToLabel() {
        try {
            if (labelInfoService.groupToLabel()) {
                return write(200, "转换成功");
            } else {
                return error(-1, "转换失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return error(-1, "转换失败");
        }
    }
}
